@@ -3,10 +3,7 @@ import logging
 from collections import namedtuple
 
 from django.db import transaction
-from django.db.models import (
-    Max,
-    Q,
-)
+from django.db.models import Max, Q
 from django.utils import timezone
 
 from mongoengine.queryset.visitor import Q as mongo_Q
@@ -119,37 +116,35 @@ def pre_migrate_content(content_model, mutable_type, lazy_type, premigrate_hook)
         state=TASK_STATES.RUNNING)
     pulp2detail_pb.save()
     existing_count = 0
-    fields = set(['id', '_storage_path', '_last_updated', '_content_type_id'])
-    if hasattr(content_model.pulp2, 'downloaded'):
-        fields.add('downloaded')
 
     if mutable_type:
         pulp2_content_ids = [
-            c.id for c in mongo_content_qs.only('id').no_cache().batch_size(batch_size)
+            c['_id'] for c in mongo_content_qs.only('id').no_cache().as_pymongo()
         ]
         # This is a mutable content type. Query for the existing pulp2content.
         # If any was found, it means that the migrated content is older than the incoming.
         # Delete outdated migrated pulp2content and create a new pulp2content
         outdated = Pulp2Content.objects.filter(pulp2_id__in=pulp2_content_ids)
-        if outdated:
+        if outdated.exists():
             pulp2mutatedcontent.extend(pulp2_content_ids)
         outdated.delete()
 
-    for i, record in enumerate(mongo_content_qs.only(*fields).no_cache().batch_size(batch_size)):
+    mongo_fields = set(['id', '_storage_path', '_last_updated', '_content_type_id'])
+    if hasattr(content_model.pulp2, 'downloaded'):
+        mongo_fields.add('downloaded')
+    for i, record in enumerate(mongo_content_qs.only(*mongo_fields).no_cache()):
         if record._last_updated == last_updated:
             # corner case - content with the last``last_updated`` date might be pre-migrated;
             # check if this content is already pre-migrated
             migrated = Pulp2Content.objects.filter(pulp2_last_updated=last_updated,
                                                    pulp2_id=record.id)
-            if migrated:
+            if migrated.exists():
                 existing_count += 1
 
                 # it has to be updated here and not later, in case all items were migrated before
                 # and no new content will be saved.
                 pulp2content_pb.total -= 1
-                pulp2content_pb.save()
                 pulp2detail_pb.total -= 1
-                pulp2detail_pb.save()
                 continue
 
         downloaded = record.downloaded if hasattr(record, 'downloaded') else False
@@ -160,28 +155,30 @@ def pre_migrate_content(content_model, mutable_type, lazy_type, premigrate_hook)
             # way to have unique records for those.
             content_relations = Pulp2RepoContent.objects.filter(
                 pulp2_unit_id=record.id,
-                pulp2_content_type_id=record._content_type_id)
-            for relation in content_relations:
-                pulp2_repo = relation.pulp2_repository
-                if not pulp2_repo.not_in_plan:
-                    item = Pulp2Content(pulp2_id=record.id,
-                                        pulp2_content_type_id=record._content_type_id,
-                                        pulp2_last_updated=record._last_updated,
-                                        pulp2_storage_path=record._storage_path,
-                                        downloaded=downloaded,
-                                        pulp2_repo=pulp2_repo)
-                    _logger.debug(
-                        'Add content item to the list to migrate: {item}'.format(item=item))
-                    pulp2content.append(item)
-                    pulp2content_pb.total += 1
-                    pulp2detail_pb.total += 1
+                pulp2_content_type_id=record._content_type_id,
+                pulp2_repository__not_in_plan=False,
+            ).select_related(
+                'pulp2_repository'
+            ).only(
+                'pulp2_repository'
+            )
+            for relation in content_relations.iterator():
+                item = Pulp2Content(pulp2_id=record.id,
+                                    pulp2_content_type_id=record._content_type_id,
+                                    pulp2_last_updated=record._last_updated,
+                                    pulp2_storage_path=record._storage_path,
+                                    downloaded=downloaded,
+                                    pulp2_repo=relation.pulp2_repository)
+                _logger.debug(
+                    'Add content item to the list to migrate: {item}'.format(item=item))
+                pulp2content.append(item)
+                pulp2content_pb.total += 1
+                pulp2detail_pb.total += 1
 
             # total needs to be adjusted, proper counting happened in the loop above,
             # so we subtract one because this content is also a part of initial 'total' counter.
             pulp2content_pb.total -= 1
             pulp2detail_pb.total -= 1
-            pulp2content_pb.save()
-            pulp2detail_pb.save()
         else:
             item = Pulp2Content(pulp2_id=record.id,
                                 pulp2_content_type_id=record._content_type_id,
@@ -211,6 +208,9 @@ def pre_migrate_content(content_model, mutable_type, lazy_type, premigrate_hook)
             pulp2content.clear()
             existing_count = 0
 
+    pulp2content_pb.save()
+    pulp2detail_pb.save()
+
     if pulp2mutatedcontent:
         # when we flip the is_migrated flag to False, we base this decision on the last_unit_added
         # https://github.com/pulp/pulp-2to3-migration/blob/master/pulp_2to3_migration/app/pre_migration.py#L279  # noqa
@@ -218,15 +218,10 @@ def pre_migrate_content(content_model, mutable_type, lazy_type, premigrate_hook)
         # in pulp2 sync and copy cases of updated errata are not covered
         # only when uploading errata last_unit_added is updated on all the repos that contain it
         mutated_content = Pulp2RepoContent.objects.filter(pulp2_unit_id__in=pulp2mutatedcontent)
-        repo_to_update_ids = set(mutated_content.values_list('pulp2_repository_id', flat=True))
-        repos_to_update = []
-        for pulp2repo in Pulp2Repository.objects.filter(pk__in=repo_to_update_ids):
-            pulp2repo.is_migrated = False
-            repos_to_update.append(pulp2repo)
+        repo_to_update_ids = mutated_content.values_list(
+            'pulp2_repository_id', flat=True).distinct()
+        Pulp2Repository.objects.filter(pk__in=repo_to_update_ids).update(is_migrated=False)
 
-        Pulp2Repository.objects.bulk_update(objs=repos_to_update,
-                                            fields=['is_migrated'],
-                                            batch_size=1000)
     if lazy_type:
         pre_migrate_lazycatalog(content_type)
 
@@ -246,25 +241,25 @@ def pre_migrate_lazycatalog(content_type):
     Args:
         content_type: A content type for which LCE should be pre-migrated
     """
-    batch_size = 10000
+    batch_size = 5000
     pulp2lazycatalog = []
 
     mongo_lce_qs = LazyCatalogEntry.objects(unit_type_id=content_type)
     total_lce = mongo_lce_qs.count()
-    for i, lce in enumerate(mongo_lce_qs.batch_size(batch_size)):
-        item = Pulp2LazyCatalog(pulp2_importer_id=lce.importer_id,
-                                pulp2_unit_id=lce.unit_id,
-                                pulp2_content_type_id=lce.unit_type_id,
-                                pulp2_storage_path=lce.path,
-                                pulp2_url=lce.url,
-                                pulp2_revision=lce.revision,
+    for i, lce in enumerate(mongo_lce_qs.batch_size(batch_size).as_pymongo().no_cache()):
+        item = Pulp2LazyCatalog(pulp2_importer_id=lce['importer_id'],
+                                pulp2_unit_id=lce['unit_id'],
+                                pulp2_content_type_id=lce['unit_type_id'],
+                                pulp2_storage_path=lce['path'],
+                                pulp2_url=lce['url'],
+                                pulp2_revision=lce['revision'],
                                 is_migrated=False)
         pulp2lazycatalog.append(item)
 
         save_batch = (i and not (i + 1) % batch_size or i == total_lce - 1)
         if save_batch:
             Pulp2LazyCatalog.objects.bulk_create(pulp2lazycatalog, ignore_conflicts=True)
-            pulp2lazycatalog = []
+            pulp2lazycatalog.clear()
 
 
 def pre_migrate_all_without_content(plan, type_to_repo_ids, repo_id_to_type):
@@ -537,10 +532,10 @@ def pre_migrate_repocontent(repo):
         return
 
     repocontent = []
-    for repocontent_data in mongo_repocontent_qs.only('unit_id',
-                                                      'unit_type_id'):
-        item = Pulp2RepoContent(pulp2_unit_id=repocontent_data.unit_id,
-                                pulp2_content_type_id=repocontent_data.unit_type_id,
+    for repocontent_data in mongo_repocontent_qs.\
+            only('unit_id', 'unit_type_id').as_pymongo().no_cache():
+        item = Pulp2RepoContent(pulp2_unit_id=repocontent_data['unit_id'],
+                                pulp2_content_type_id=repocontent_data['unit_type_id'],
                                 pulp2_repository=repo)
         repocontent.append(item)
 
